﻿using System.Collections;
using System.Collections.Generic;
using UnityEditor;
using UnityEngine;
using LitJson;
using UnityEditor.AnimatedValues;
using System;

#if UNITY_EDITOR
namespace JXSoft {
    public class JsonFileLoaderEditor : EditorWindow
    {
        public TextAsset JsonFile;
        AnimBool m_ShowExtraFields;
        private JsonData jsonData;
        private List<bool> hoverArrowGroup;
        private Vector2 scrollRect;


        [MenuItem("JXSoft/Loader/JsonFileLoader")]
        static void Init()
        {
            var window = GetWindow<JsonFileLoaderEditor>();
            window.Show();
        }
        void OnEnable()
        {
            m_ShowExtraFields = new AnimBool(true);
            m_ShowExtraFields.valueChanged.AddListener(Repaint);
        }

        void OnGUI()
        {
            JsonFile = (TextAsset)EditorGUILayout.ObjectField("JsonFile", JsonFile, typeof(TextAsset), true);
            if (GUILayout.Button("AnalyzeJsonFile"))
            {
                if (JsonFile != null)
                {
                    jsonData = JsonMapper.ToObject(JsonFile.text);
                }
            }
            if (GUILayout.Button("Clear")) {
                jsonData.Clear();
            }
            scrollRect = EditorGUILayout.BeginScrollView(scrollRect);
            if (jsonData!=null) {
                loadData(jsonData);
            }
            EditorGUILayout.EndScrollView();
        }
        public void loadData(JsonData data) {
            foreach (string key in data.Keys)
            {
                if (data[key].IsObject)
                {
                    EditorGUILayout.LabelField(key, EditorStyles.boldLabel);
                    EditorGUILayout.BeginVertical("Label");
                    loadData(data[key]);
                    EditorGUILayout.EndVertical();
                }else if (data[key].IsArray) {
                    EditorGUILayout.LabelField(key, EditorStyles.boldLabel);
                    loadArray(data[key]);
                }
                else
                {
                    EditorGUILayout.TextField(key, data[key].ToString());
                }
            }
        }
        public void loadArray(JsonData data) {
            foreach (JsonData v in data)
            {
                EditorGUILayout.BeginVertical("Label");
                if (v.IsArray)
                {
                    EditorGUILayout.LabelField(v.ToString(), EditorStyles.boldLabel);
                    loadArray(v);
                }
                else {
                    EditorGUILayout.TextField("", v.ToString());
                }
                EditorGUILayout.EndVertical();
            }
        }

        public void generateCode(JsonData data) { 
            
        }
    }

}

#endif
