﻿using System;
using System.IO.Ports;
using System.Threading;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using QFrameworkCP;


namespace ModbusListener {
    public class ModbusToolsArchitecture : Architecture<ModbusToolsArchitecture>
    {
        protected override void Init()
        {
            this.RegisterModel(new ModbusTestModel());
        }
    }
    public class ModbusTestModel : AbstractModel
    {
        public int functionCode;
        public ModbusData mdata;
        protected override void OnInit()
        {
        }
    }
    public class ModbusListener :MonoBehaviour,IController{
        private List<int> sendData = new List<int>();
        private Thread readPortData;
        private SerialPort serialPort = new SerialPort("COM4", 9600, Parity.None, 8, StopBits.One);
        //是否获取串口信息
        private bool getSerialMsg = false;
        private void Start()
        {
            openSerialPort();
        }
        void Update()
        {
            updateSwitchEventWithoutDevices();
            if (this.GetModel<ModbusTestModel>().mdata != null && getSerialMsg)
            {
                updateSwitchEvent(this.GetModel<ModbusTestModel>().mdata);
                Debug.Log(this.GetModel<ModbusTestModel>().mdata.toString());
                getSerialMsg = false;
            }
        }

        public void receivedData(byte[] data) { 

            
        }
        private void readData()
        {
            try
            {
                while (serialPort != null && serialPort.IsOpen)
                {
                    LogData();
                }
            }
            catch (Exception e)
            {
                Debug.LogWarning(e);
                throw;
            }
        }
        private void LogData()
        {
            int s = 0;
            int count = 8;
            List<int> dataCode = new List<int>();
            //以行的模式读取串口数据
            while (count > 0)
            {
                s = serialPort.ReadByte();
                dataCode.Add(s);
                count--;
                //print(s); //打印读取到的每一行数据
            }
            this.GetModel<ModbusTestModel>().mdata = new ModbusData(dataCode);
            this.GetModel<ModbusTestModel>().functionCode = this.GetModel<ModbusTestModel>().mdata.addressCode;
            getSerialMsg = true;
        }
        private void openSerialPort()
        {
            string ModbusSerial = "COM4";
            try
            {
                serialPort = new SerialPort(ModbusSerial, 9600, Parity.None, 8, StopBits.One);
                if (!serialPort.IsOpen)
                {
                    serialPort.Open();
                }
                readPortData = new Thread(new ThreadStart(readData));
                readPortData.IsBackground = true;
                readPortData.Start();
            }
            catch (Exception e)
            {
                Debug.LogWarning(e);
                throw;
            }
        }

        public IArchitecture GetArchitecture()
        {
            return ModbusToolsArchitecture.Interface;
        }
        private void OnDestroy()
        {
            if (serialPort != null && serialPort.IsOpen)
            {
                serialPort.Close();
            }
        }
        #region 更新无线开关状态
        public void updateSwitchEvent(ModbusData mdata)
        {
            switch (mdata.addressCode)
            {
                case 100:
                    //报告完成
                    Debug.Log("100Switch");
                    break;
                case 101:
                    //设置警告
                    Debug.Log("101Switch");
                    break;
                case 102:
                    Debug.Log("102Switch");
                    break;
                default:
                    break;
            }
        }

        public void updateSwitchEventWithoutDevices()
        {
            int addressCode = 0;
            int valueCode = 0;
            if (Input.GetKeyDown(KeyCode.F1))
            {
                addressCode = 100;
                valueCode = 1;
            }
            else if (Input.GetKeyDown(KeyCode.F2))
            {
                addressCode = 101;
                valueCode = 1;
            }
            else if (Input.GetKeyDown(KeyCode.F3))
            {
                addressCode = 102;
                valueCode = 1;
            }
            else if (Input.GetKeyUp(KeyCode.F1))
            {
                addressCode = 100;
                valueCode = 0;
            }
            else if (Input.GetKeyUp(KeyCode.F2))
            {
                addressCode = 101;
                valueCode = 0;
            }
            else if (Input.GetKeyUp(KeyCode.F3))
            {
                addressCode = 102;
                valueCode = 0;
            }
            else
            {
                return;
            }
            this.GetModel<ModbusTestModel>().mdata = new ModbusData(addressCode, valueCode);
            this.GetModel<ModbusTestModel>().functionCode = this.GetModel<ModbusTestModel>().mdata.addressCode;
            getSerialMsg = true;
        }

        #endregion
    }



    /// <summary>
    /// 发送Modbus命令
    /// </summary>
    /// <typeparam name="TModbusData">Modbus数据类</typeparam>
    public class SendModbusDataCommand<TModbusData> : AbstractCommand where TModbusData : IModbusData
    {
        private SerialPort port;
        private TModbusData modbusData;
        public SendModbusDataCommand(SerialPort port, TModbusData modbusData) {
            this.port = port;
            this.modbusData = modbusData;
        }
        protected override void OnExecute()
        {
            if (port != null)
            {
                port.Write(modbusData.getBytes(), 0, modbusData.getBytes().Length);
            }
            else {
                Debug.LogWarning("Please Open Your SerialPort!");
            }
            //throw new System.NotImplementedException();
        }
    }

    /// <summary>
    /// 接收Modbus指令
    /// </summary>
    public class ReciveModbusDataCommand : AbstractCommand
    {
        private SerialPort port;
        private Action<byte[]> recivedEvent;
        private bool isGetData;
        private Thread readPortData;
        private int dataLength;
        /// <summary>
        /// 构造
        /// </summary>
        /// <param name="port">串口</param>
        /// <param name="recivedEvent">接收回调</param>
        /// <param name="dataLength">数据长度</param>
        public ReciveModbusDataCommand(SerialPort port, Action<byte[]> recivedEvent,int dataLength) {
            this.port = port;
            this.recivedEvent = recivedEvent;
            this.dataLength = dataLength;
            this.isGetData = false;
        }

        protected override void OnExecute()
        {
            //throw new NotImplementedException();
            readPortData = new Thread(new ThreadStart(readData));
            readPortData.IsBackground = true;
            readPortData.Start();
        }
        private void readData()
        {
            try
            {
                while (port != null && port.IsOpen)
                {
                    LogData();
                }
            }
            catch (Exception e)
            {
                Debug.LogWarning(e);
                throw;
            }
        }
        private void LogData()
        {
            int s = 0;
            int count = dataLength;
            byte[] dataCode = new byte[dataLength];
            //以行的模式读取串口数据
            while (count > 0)
            {
                s = port.ReadByte();
                dataCode[count] = Convert.ToByte(s);
                count--;
                //print(s); //打印读取到的每一行数据
            }
            recivedEvent(dataCode);
        }
    }


    public interface IModbusData {
        void setBytes(byte[] bytes);
        byte[] getBytes();
    }

    public class ModbusData : IModbusData
    {
        //该构造只包含写单个跟读单个，后续会修改支持写多个
        //机器码0-255，占两位
        public int mechineNo;
        //功能码，占两位
        public int functionCode;
        //地址码0-65535，占四位
        public int addressCode;
        //数据码0-65535，占四位
        public int valueCode;
        //校验码0-65535，占四位（CRC-16/MODBUS）
        //另附在线计算器 http://www.ip33.com/crc.html
        public int verifiedCode;
        //byte数组格式
        public byte[] originByteData;

        public ModbusData(List<int> code)
        {
            mechineNo = code[0];
            functionCode = code[1];
            addressCode = code[2] + code[3];
            valueCode = code[4] * 256 + code[5];
            verifiedCode = code[6] * 256 + code[7];

        }

        public ModbusData(byte[] code)
        {
            mechineNo = code[0];
            functionCode = code[1];
            addressCode = code[2] + code[3];
            valueCode = code[4] * 256 + code[5];
            verifiedCode = code[6] * 256 + code[7];
            setBytes(code);
        }

        public ModbusData(int address, int valueCode)
        {
            this.addressCode = address;
            this.valueCode = valueCode;
        }

        public byte[] getBytes()
        {
            return originByteData;
            //throw new NotImplementedException();
        }

        public void setBytes(byte[] bytes)
        {
            originByteData = bytes;
        }

        public string toString()
        {
            string valueStr = "";
            valueStr = "mechineNo:" + mechineNo + "|-|functionCode:" + functionCode + "|-|addressCode:" + addressCode + "|-|valueCode:" + valueCode + "|-|verifiedCode:" + verifiedCode;
            return valueStr;
        }
    }
}
